/*- Globals.cpp ---------------------------------------------------------------*\
|
|   Contains:   Global functions.
|
|   Written by: Tom Plunket, tomas@teleport.com:  TJP
|
|   Copyright:  (c) 1997, 1998 by Tom Plunket, all rights reserved.
|
|   File history:
|       12/31/97    TJP
|       File created.
|
|        1/21/98    TJP
|       Hacked comments out from FirstMod, made SkeletonClassDesc.
|
|        1/28/98    TJP
|       Added GetNode(), GetNodes(), and the NodeRetrievalDEP class.
|
\*-----------------------------------------------------------------------------*/
#include <Max.h>
#include "Skeleton.h"
#include "Globals.h"

//---------------------------
// Global things
HINSTANCE hInstance = NULL;  // This is a handle that is used to get at resources, among other things.

//---------------------------
// ClassDescs
//
class SkeletonClassDesc : public ClassDesc
{
    int IsPublic()            { return 1; }
    void *Create(BOOL loading){ return new Skeleton; }
    const TCHAR* ClassName()  { return SKELETON_NAME; }
    SClass_ID SuperClassID()  { return WSM_CLASS_ID; }
    Class_ID ClassID()        { return SKELETON_CLASS_ID; }
    const TCHAR* Category()   { return TOMS_PLUGINS; }
};
static SkeletonClassDesc cdSkel;

//---------------------------
// LibStuff
__declspec( dllexport ) ULONG LibVersion()
{
    return VERSION_3DSMAX;
}

__declspec( dllexport ) const TCHAR *LibDescription()
{
    return _T("Skeleton Mesh Deformer");
}

__declspec( dllexport ) int LibNumberClasses()
{
    return 1;
}

__declspec( dllexport ) ClassDesc* LibClassDesc(int i)
{
	switch(i)
    {
    case 0:
        return &cdSkel;
        
    default:
        return NULL;
    }
}



//---------------------------
// DllMain
BOOL WINAPI DllMain(HINSTANCE hinstDLL,ULONG fdwReason,LPVOID lpvReserved)
{
    static int controlsInit = FALSE;

	if ( !controlsInit )
    {
        controlsInit = TRUE;

        // Grab the handle to the instance.  This allows us to use the
        // rollup windows and get at our resources.
        hInstance = hinstDLL;

        // InitCustomControls is needed to use MAX's custom control stuff.
        InitCustomControls(hInstance);

        // InitCommonControls is needed to use the Windows controls.
        // REMEMBER to include comctl32.lib in your lib file list.
        InitCommonControls();
    }

    return TRUE;
}

//-------------------------------------
// This class is used to get the list
// of nodes that reference an object
// when EnumDependents is called on
// that object.
class NodeRetrievalDEP : public DependentEnumProc, public INodeTab
{
    int proc(ReferenceMaker *rmaker)
    {
        if (rmaker->SuperClassID() == BASENODE_CLASS_ID)
            Append(1, (INode**)&rmaker, 5);
        
        return 0;
    }
};

//---------------------------
// GetNode
//
// Returns the node that references the
// passed-in object, or NULL if more or
// less than one node reference the object.
INode* GetNode(RefTargetHandle obj)
{
    NodeRetrievalDEP nrdep;
    INode* retVal = NULL;
    int i;

    obj->EnumDependents(&nrdep);

    for (i = 0 ; i < nrdep.Count() ; ++i)
    {
        if (nrdep[i]->EvalWorldState(0).obj == obj)
        {
            if (retVal == NULL)
                retVal = nrdep[i];
//            else
//                return NULL;
        }
    }

    return retVal;
}

//---------------------------
// GetNodes
//
// Fills in the INodeTab with all of the
// nodes that reference the passed-in
// object, and returns the number of
// nodes therein.
int GetNodes(RefTargetHandle obj, INodeTab* nt)
{
    NodeRetrievalDEP nrdep;

    obj->EnumDependents(&nrdep);
    *nt = nrdep;

    return nt->Count();
}
/*
BOOL CALLBACK BaseDlgProc(HWND h, UINT m, WPARAM w, LPARAM l)
{
    CDlgClassBase* c;

    if (m == WM_INITDIALOG)
    {
        SetWindowLong(h, GWL_USERDATA, l);
        c = (CDlgClassBase*)l;
    }
    else
        c = (CDlgClassBase*)GetWindowLong(h, GWL_USERDATA);

    return c->DlgProc(h, m, w, l);
}
*/