/*- Skeleton.h ----------------------------------------------------------------*\
|
|   Contains:   Definition for Modifier-derived class.
|
|   Written by: Tom Plunket, tomas@teleport.com:  TJP
|
|   Copyright:  (c) 1998 by Tom Plunket, all rights reserved
|
|   File history:
|        1/21/98    TJP
|       File created.
|
\*-----------------------------------------------------------------------------*/
#ifndef __SKELETON_H

#include <Max.h>
#include "Globals.h"

#define SKELETON_CLASS_ID   (Class_ID(0x74305eb3, 0x02a5108e))
#define SKELETON_NAME       (_T("Skeleton"))

class Skeleton : public WSModifier//, public CDlgClassBase
{
public:
    Skeleton();
   ~Skeleton();

    //-------------
    // mine
    //BOOL DlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
    void GetBones();
    void InitRollup(HWND hWnd);
    void DestroyRollup();
    void SpinnerChange(short index);

    //-------------
    // Modifier
//    TSTR GetName() { return SKELETON_NAME; }
    Class_ID InputType() { return defObjectClassID; }
    ChannelMask ChannelsUsed() { return GEOM_CHANNEL; }
    ChannelMask ChannelsChanged() { return GEOM_CHANNEL; }
    void NotifyInputChanged(Interval changeInt, PartID partID, RefMessage message, ModContext *mc);
    void ModifyObject(TimeValue t, ModContext &mc, ObjectState* os, INode *node);
    Interval LocalValidity(TimeValue t);
    BOOL DependOnTopology(ModContext &mc) { return FALSE; }

    //-------------
    // BaseObject
    TCHAR *GetObjectName() { return SKELETON_NAME; }
    CreateMouseCallBack* GetCreateMouseCallBack() { return NULL; }
    BOOL ChangeTopology() { return FALSE; }

    //-------------
    // ReferenceTarget
    RefTargetHandle Clone(RemapDir &remap = NoRemap());

    //-------------
    // ReferenceMaker
    RefResult NotifyRefChanged(Interval changeInt, RefTargetHandle hTarget, PartID& partID, RefMessage message);
    int NumRefs() { return boneList.Count() + 1; }
    RefTargetHandle GetReference(int i) { return (i < boneList.Count() ? boneList[i] : NULL); }
    void SetReference(int i, RefTargetHandle rtarg);
    IOResult Save(ISave *isave);
    IOResult Load(ILoad *iload);

    //-------------
    // Animatable
    void GetClassName(TSTR& s) { s = SKELETON_NAME; }
    Class_ID ClassID() { return SKELETON_CLASS_ID; }
    void DeleteThis() { delete this; }
    void BeginEditParams(IObjParam *ip, ULONG flags, Animatable *prev);
    void EndEditParams(IObjParam *ip, ULONG flags, Animatable *next);
/*
    int NumSubs();
    Animatable* SubAnim(int i);
    TSTR SubAnimName(int i);
    ParamDimension* GetParamDimension(int i);
    int SubNumToRefNum(int subNum);
    int HasSubElements(int type=0);
*/
    // We are a subclass of ourself, and a subclass of whatever OSModifier allows, too.
    BOOL IsSubClassOf(Class_ID classID) { return (classID == SKELETON_CLASS_ID) ? TRUE : WSModifier::IsSubClassOf(classID); }

private:
    INodeTab boneList;
    static Interface* maxInterface;
    static HWND hPanel;
    static HWND comboBox;
    static ISpinnerControl* falloffSpinner;

    void FillComboBox();
    
    unsigned long pendingOperations;
    double** influences;
    float falloff;
};

//-------------------------------------
// Exported from Influence.cpp
double** Influence(INode* defNode, ObjectState* defOs, INodeTab& bones, float falloff);

#endif // ndef __SKELETON_H
